@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2012-2015 by Christoph Reller. All rights reserved.
+# Copyright (c) 2012-2020 by Christoph Reller. All rights reserved.
 
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -36,9 +36,10 @@
     See README.rst file for details
 
     Author: Christoph Reller <christoph.reller@gmail.com>
-    Version: 0.4.5
 """
 
+__version__ = '0.4.9'
+
 import contextlib
 import tempfile
 import posixpath
@@ -57,8 +58,7 @@ except ImportError:
     from sha import sha
 
 from docutils import nodes, utils
-from docutils.parsers.rst import directives
-from docutils.parsers.rst import Directive
+from docutils.parsers.rst import Directive, directives
 
 from sphinx.errors import SphinxError
 try:
@@ -97,10 +97,7 @@ def system(command, builder, outfile=None):
     except OSError as err:
         if err.errno != ENOENT:   # No such file or directory
             raise
-        builder.warn('%s command cannot be run' % binary)
-        builder.warn(err)
-        builder._tikz_warned = True
-        return None
+        raise TikzExtError('!%s command cannot be run' % binary)
     stdout, stderr = process.communicate()
     if process.returncode != 0:
         builder._tikz_warned = True
@@ -141,6 +138,7 @@ class TikzDirective(Directive):
 
     def run(self):
         node = tikz()
+        captionstr = ''
 
         if 'include' in self.options:
             node['include'] = self.options['include']
@@ -157,16 +155,14 @@ class TikzDirective(Directive):
                 return [self.state.document.reporter.warning(
                     'External Tikz file %r not found or reading '
                     'it failed' % filename, line=self.lineno)]
-            node['caption'] = ''
             if self.arguments:
-                node['caption'] = '\n'.join(self.arguments)
+                captionstr = '\n'.join(self.arguments)
         else:
             if not self.content:
-                node['caption'] = ''
                 node['tikz'] = '\n'.join(self.arguments)
             else:
                 node['tikz'] = '\n'.join(self.content)
-                node['caption'] = '\n'.join(self.arguments)
+                captionstr = '\n'.join(self.arguments)
 
         node['libs'] = self.options.get('libs', '')
         if 'stringsubst' in self.options:
@@ -177,6 +173,11 @@ class TikzDirective(Directive):
             return [self.state_machine.reporter.warning(
                     'Ignoring "tikz" directive without content.',
                     line=self.lineno)]
+
+        # If we have a caption, add it as a child node.
+        if captionstr:
+            node += nodes.caption(captionstr, '', nodes.Text(captionstr))
+
         return [node]
 
 DOC_HEAD = r'''
@@ -215,9 +216,10 @@ def cleanup_tikzcode(self, node):
 
 
 def render_tikz(self, node, libs='', stringsubst=False):
+    # must use unique filenames for all tmpfiles to support sphinx -j
     tikz = cleanup_tikzcode(self, node)
-    hashkey = tikz.encode('utf-8')
-    fname = 'tikz-%s.%s' % (sha(hashkey).hexdigest(),
+    shasum = sha(tikz.encode('utf-8')).hexdigest()
+    fname = 'tikz-%s.%s' % (shasum,
                             OUT_EXTENSION[self.builder.config.tikz_proc_suite])
     relfn = posixpath.join(self.builder.imgpath, fname)
     outfn = path.join(self.builder.outdir, '_images', fname)
@@ -237,19 +239,21 @@ def render_tikz(self, node, libs='', stringsubst=False):
 
     with changedir(self.builder._tikz_tempdir):
 
-        tf = open('tikz.tex', 'wb')
+        tf = open('tikz-%s.tex' % shasum, 'wb')
         tf.write(latex)
         tf.close()
 
         system([self.builder.config.latex_engine, '--interaction=nonstopmode',
-                'tikz.tex'],
+                'tikz-%s.tex' % shasum],
                self.builder)
 
+        resolution = str(self.builder.config.tikz_resolution)
+
         if self.builder.config.tikz_proc_suite in ['ImageMagick', 'Netpbm']:
 
-            system(['pdftoppm', '-r', '400', 'tikz.pdf',
-                    'tikz'], self.builder)
-            ppmfilename = glob('tikz*.ppm')[0]
+            system(['pdftoppm', '-r', resolution, 'tikz-%s.pdf' % shasum, 'tikz-%s' %
+                    shasum], self.builder)
+            ppmfilename = glob('tikz-%s*.ppm' % shasum)[0]
 
             if self.builder.config.tikz_proc_suite == "ImageMagick":
                 if self.builder.config.tikz_transparent:
@@ -274,11 +278,11 @@ def render_tikz(self, node, libs='', stringsubst=False):
                 device = "pngalpha"
             else:
                 device = "png256"
-            system([ghostscript, '-dBATCH', '-dNOPAUSE',
-                    '-sDEVICE=%s' % device, '-sOutputFile=%s' % outfn,
-                    '-r120x120', '-f', 'tikz.pdf'], self.builder)
+            system([ghostscript, '-dBATCH', '-dNOPAUSE', '-sDEVICE=%s' % device,
+                    '-sOutputFile=%s' % outfn, '-r' + resolution + 'x' + resolution,
+                    '-f', 'tikz-%s.pdf' % shasum], self.builder)
         elif self.builder.config.tikz_proc_suite == "pdf2svg":
-            system(['pdf2svg', 'tikz.pdf', outfn], self.builder)
+            system(['pdf2svg', 'tikz-%s.pdf' % shasum, outfn], self.builder)
         else:
             self.builder._tikz_warned = True
             raise TikzExtError('Error (tikz extension): Invalid configuration '
@@ -293,20 +297,14 @@ def html_visit_tikzinline(self, node):
     try:
         fname = render_tikz(self, node, libs)
     except TikzExtError as exc:
-        info = str(exc)[str(exc).find('!'):-1]
+        info = str(exc)[str(exc).find('!'):].replace('\\n', '\n')
         sm = nodes.system_message(info, type='WARNING', level=2,
                                   backrefs=[], source=node['tikz'])
         sm.walkabout(self)
-        self.builder.warn('display latex %r: \n' % node['tikz'] + str(exc))
-        raise nodes.SkipNode
-    if fname is None:
-        # something failed -- use text-only as a bad substitute
-        self.body.append('<span class="math">%s</span>' %
-                         self.encode(node['tikz']).strip())
     else:
-        self.body.append('<img class="math" src="%s" alt="%s"/>' %
+        self.body.append('<img src="%s" alt="%s"/>' %
                          (fname, self.encode(node['tikz']).strip()))
-        raise nodes.SkipNode
+    raise nodes.SkipNode
 
 
 def html_visit_tikz(self, node):
@@ -315,26 +313,19 @@ def html_visit_tikz(self, node):
     try:
         fname = render_tikz(self, node, libs, node['stringsubst'])
     except TikzExtError as exc:
-        info = str(exc)[str(exc).find('!'):-1]
+        info = str(exc)[str(exc).find('!'):].replace('\\n', '\n')
         sm = nodes.system_message(info, type='WARNING', level=2,
                                   backrefs=[], source=node['tikz'])
         sm.walkabout(self)
-        self.builder.warn('display latex %r: \n' % node['tikz'] + str(exc))
-        raise nodes.SkipNode
-    if fname is None:
-        # something failed -- use text-only as a bad substitute
-        self.body.append('<span class="math">%s</span>' %
-                         self.encode(node['tikz']).strip())
     else:
         self.body.append(self.starttag(node, 'div', CLASS='figure'))
         self.body.append('<p>')
         self.body.append('<img src="%s" alt="%s" /></p>\n' %
                          (fname, self.encode(node['tikz']).strip()))
-        if node['caption']:
-            self.body.append('<p class="caption">%s</p>' %
-                             self.encode(node['caption']).strip())
-        self.body.append('</div>')
-        raise nodes.SkipNode
+
+
+def html_depart_tikz(self, node):
+    self.body.append('</div>')
 
 
 def latex_visit_tikzinline(self, node):
@@ -356,16 +347,25 @@ def latex_visit_tikzinline(self, node):
 
 def latex_visit_tikz(self, node):
     tikz = cleanup_tikzcode(self, node)
-    if node['caption']:
-        latex = '\\begin{figure}[htp]\\centering' + tikz \
-                + '\\caption{' + self.encode(node['caption']).strip() \
-                + '}\\end{figure}'
+
+    # Have a caption: enclose in a figure environment.
+    if any(isinstance(child, nodes.caption) for child in node.children):
+        self.body.append('\\begin{figure}[htp]\\centering\\capstart' + tikz)
+
+    # No caption: place in a center environment.
     else:
-        latex = '\\begin{center}' + tikz + '\\end{center}'
-    self.body.append(latex)
+        self.body.append('\\begin{center}' + tikz + '\\end{center}')
 
 
-def depart_tikz(self, node):
+def latex_depart_tikz(self, node):
+    # If we have a caption, we need to add a label for any cross-referencing
+    # and then close the figure environment.
+    if any(isinstance(child, nodes.caption) for child in node.children):
+        self.body.append(self.hypertarget_to(node))
+        self.body.append('\\end{figure}')
+
+
+def depart_tikzinline(self, node):
     pass
 
 
@@ -391,12 +391,12 @@ def builder_inited(app):
         sty.write(r"\RequirePackage{amsmath}" + "\n")
         sty.write(r"\RequirePackage{amsfonts}" + "\n")
         sty.write(r"\RequirePackage{pgfplots}" + "\n")
-        sty.write(app.builder.config.tikz_latex_preamble + "\n")
         tikzlibs = app.builder.config.tikz_tikzlibraries
         tikzlibs = tikzlibs.replace(' ', '')
         tikzlibs = tikzlibs.replace('\t', '')
         tikzlibs = tikzlibs.strip(', ') + "\n"
         sty.write(r"\usetikzlibrary{%s}" % tikzlibs)
+        sty.write(app.builder.config.tikz_latex_preamble + "\n")
         sty.close()
 
         app.builder.config.latex_additional_files.append(sty_path)
@@ -425,12 +425,12 @@ def which(program):
 
 
 def setup(app):
-    app.add_node(tikz,
-                 html=(html_visit_tikz, depart_tikz),
-                 latex=(latex_visit_tikz, depart_tikz))
+    app.add_enumerable_node(tikz, 'figure',
+                 html=(html_visit_tikz, html_depart_tikz),
+                 latex=(latex_visit_tikz, latex_depart_tikz))
     app.add_node(tikzinline,
-                 html=(html_visit_tikzinline, depart_tikz),
-                 latex=(latex_visit_tikzinline, depart_tikz))
+                 html=(html_visit_tikzinline, depart_tikzinline),
+                 latex=(latex_visit_tikzinline, depart_tikzinline))
     app.add_role('tikz', tikz_role)
     app.add_directive('tikz', TikzDirective)
     app.add_config_value('tikz_latex_preamble', '', 'html')
@@ -446,6 +446,13 @@ def setup(app):
             if not which('pnmcrop'):
                 suite = 'Netpbm'
     app.add_config_value('tikz_proc_suite', suite, 'html')
+    app.add_config_value('tikz_resolution', 184, 'html')
 
     app.connect('build-finished', cleanup_tempdir)
     app.connect('builder-inited', builder_inited)
+
+    return {
+        'version': __version__,
+        'parallel_read_safe': True,
+        'parallel_write_safe': True
+    }
